@@ -50,7 +50,7 @@ typedef struct utmp UTMP;
 
 #define MAX_TTY_SIZE (sizeof (PATH_TTY_PFX) + sizeof (((UTMP*)0)->ut_line))
 
-const char *argp_program_version = "comsatd (" PACKAGE_STRING ")";
+const char *program_version = "comsatd (" PACKAGE_STRING ")";
 static char doc[] = "GNU comsatd";
 
 static struct argp_option options[] = 
@@ -139,6 +139,7 @@ main(int argc, char **argv)
   /* Native Language Support */
   mu_init_nls ();
 
+  mu_argp_init (program_version, NULL);
   mu_argp_parse (&argp, &argc, &argv, 0, comsat_argp_capa,
 		 NULL, &daemon_param);
 
@@ -286,7 +287,10 @@ comsat_daemon (int port)
 	      unsigned delay;
 
 	      delay = overflow_delay_time << (overflow_count + 1);
-	      syslog (LOG_NOTICE, _("too many requests: pausing for %u seconds"),
+	      syslog (LOG_NOTICE,
+		      ngettext ("too many requests: pausing for %u second",
+				"too many requests: pausing for %u seconds",
+				delay),
 		      delay);
 	      sleep (delay);
 	      reqcount = 0;
@@ -344,7 +348,10 @@ comsat_main (int fd)
       return 1;
     }
 
-  syslog (LOG_INFO, _("%d bytes from %s"), rdlen, inet_ntoa (sin_from.sin_addr));
+  syslog (LOG_INFO,
+	  ngettext ("%d byte from %s",
+		    "%d bytes from %s", rdlen),
+	  rdlen, inet_ntoa (sin_from.sin_addr));
 
   buffer[rdlen] = 0;
 
