@@ -1,19 +1,19 @@
-/* GNU mailutils - a suite of utilities for electronic mail
+/* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.
 
-   This program is distributed in the hope that it will be useful,
+   GNU Mailutils is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with GNU Mailutils; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -92,7 +92,7 @@ main (int argc, char **argv)
 
   if ((ret = mailbox_create_default (&mbox, mailbox_name)) != 0)
     {
-      fprintf (stderr, "could not create - %s\n", mu_errstring (ret));
+      fprintf (stderr, "could not create - %s\n", mu_strerror (ret));
       exit (2);
     }
 
@@ -107,7 +107,7 @@ main (int argc, char **argv)
   /* Open the mailbox for reading only.  */
   if ((ret = mailbox_open (mbox, MU_STREAM_RDWR)) != 0)
     {
-      fprintf (stderr, "mailbox open - %s\n", mu_errstring (ret));
+      fprintf (stderr, "mailbox open - %s\n", mu_strerror (ret));
       exit (2);
     }
 
@@ -123,17 +123,17 @@ main (int argc, char **argv)
 
       if ((ret = mailbox_get_message (mbox, i, &msg)) != 0)
         {
-          fprintf (stderr, "mailbox_get_message - %s\n", mu_errstring (ret));
+          fprintf (stderr, "mailbox_get_message - %s\n", mu_strerror (ret));
           exit (2);
         }
       if ((ret = message_size (msg, &msize)) != 0)
         {
-          fprintf (stderr, "message_size - %s\n", mu_errstring (ret));
+          fprintf (stderr, "message_size - %s\n", mu_strerror (ret));
           exit (2);
         }
       if ((ret = message_get_header (msg, &hdr)) != 0)
         {
-          fprintf (stderr, "message_get_header - %s\n", mu_errstring (ret));
+          fprintf (stderr, "message_get_header - %s\n", mu_strerror (ret));
           exit (2);
         }
       header_get_value (hdr, MU_HEADER_FROM, from, sizeof (from), NULL);
@@ -145,7 +145,7 @@ main (int argc, char **argv)
 
       if ((ret = message_get_num_parts (msg, &nparts)) != 0)
         {
-          fprintf (stderr, "message_get_num_parts - %s\n", mu_errstring (ret));
+          fprintf (stderr, "message_get_num_parts - %s\n", mu_strerror (ret));
           exit (2);
         }
       printf ("Number of parts in message - %lu\n",
@@ -178,7 +178,7 @@ message_display_parts (message_t msg, int indent)
   /* How many parts does the message has? */
   if ((ret = message_get_num_parts (msg, &nparts)) != 0)
     {
-      fprintf (stderr, "message_get_num_parts - %s\n", mu_errstring (ret));
+      fprintf (stderr, "message_get_num_parts - %s\n", mu_strerror (ret));
       exit (2);
     }
 
@@ -189,17 +189,17 @@ message_display_parts (message_t msg, int indent)
     {
       if ((ret = message_get_part (msg, j, &part)) != 0)
         {
-          fprintf (stderr, "mime_get_part - %s\n", mu_errstring (ret));
+          fprintf (stderr, "mime_get_part - %s\n", mu_strerror (ret));
           exit (2);
         }
       if ((ret = message_size (part, &msize)) != 0)
         {
-          fprintf (stderr, "message_size - %s\n", mu_errstring (ret));
+          fprintf (stderr, "message_size - %s\n", mu_strerror (ret));
           exit (2);
         }
       if ((ret = message_get_header (part, &hdr)) != 0)
         {
-          fprintf (stderr, "message_get_header - %s\n", mu_errstring (ret));
+          fprintf (stderr, "message_get_header - %s\n", mu_strerror (ret));
           exit (2);
         }
       header_get_value (hdr, MU_HEADER_CONTENT_TYPE, type, sizeof (type),
@@ -220,13 +220,13 @@ message_display_parts (message_t msg, int indent)
               ret = message_unencapsulate (part, &part, NULL);
               if (ret != 0)
                 fprintf (stderr, "message_unencapsulate - %s\n",
-                         mu_errstring (ret));
+                         mu_strerror (ret));
               break;
             }
           if ((ret = message_get_header (part, &hdr)) != 0)
             {
               fprintf (stderr, "message_get_header - %s\n",
-                       mu_errstring (ret));
+                       mu_strerror (ret));
               exit (2);
             }
           header_get_value (hdr, MU_HEADER_FROM, from, sizeof (from), NULL);
@@ -238,7 +238,7 @@ message_display_parts (message_t msg, int indent)
           if ((ret = message_get_num_parts (part, &nsubparts)) != 0)
             {
               fprintf (stderr, "mime_get_num_parts - %s\n",
-                       mu_errstring (ret));
+                       mu_strerror (ret));
               exit (2);
             }
           message_display_parts (part, indent+indent_level);
