@@ -1,19 +1,19 @@
-/* GNU mailutils - a suite of utilities for electronic mail
+/* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2001 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.
 
-   This program is distributed in the hope that it will be useful,
+   GNU Mailutils is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with GNU Mailutils; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
 
 #include "imap4d.h"
 #include <ctype.h>
@@ -268,10 +268,9 @@ fetch_fast (struct fetch_command *command, char **arg)
    Header: Date, Subject, From, Sender, Reply-To, To, Cc, Bcc, In-Reply-To,
    and Message-Id.  */
 static int
-fetch_envelope (struct fetch_command *command, char **arg)
+fetch_envelope (struct fetch_command *command, char **arg ARG_UNUSED)
 {
   int status;
-  (void)arg; /* No arguments.  */
   util_send ("%s (", command->name);
   status = fetch_envelope0 (command->msg);
   util_send (")");
@@ -290,14 +289,11 @@ fetch_flags0 (const char *prefix, message_t msg, int isuid)
     {
       struct fetch_command *fcmd = &fetch_command_table[F_UID];
       fcmd->msg = msg;
-      util_send ("(");
       fetch_uid (fcmd, NULL);
       util_send (" ");
     }
   util_send ("%s (", prefix);
   util_print_flags(attr);
-  if (isuid)
-    util_send (")");
   util_send (")");
 }
 
@@ -339,20 +335,27 @@ fetch_flags (struct fetch_command *command, char **arg)
    ;; from the given time will give the UT form.
    ;; The Universal Time zone is "+0000".  */
 static int
-fetch_internaldate (struct fetch_command *command, char **arg)
+fetch_internaldate (struct fetch_command *command, char **arg ARG_UNUSED)
 {
-  char date[128], *p;
+  char date[128];
   envelope_t env = NULL;
-  struct tm tm;
+  struct tm tm, *tmp = NULL;
   mu_timezone tz;
 
-  (void)arg; /* No arguments.  */
   message_get_envelope (command->msg, &env);
   date[0] = '\0';
-  envelope_date (env, date, sizeof (date), NULL);
-  p = date;
-  mu_parse_ctime_date_time ((const char **) &p, &tm, &tz);
-  strftime (date, sizeof (date), "%d-%b-%Y %X", &tm);
+  if (envelope_date (env, date, sizeof (date), NULL) == 0)
+    {
+      char *p = date;
+      if (mu_parse_ctime_date_time ((const char **) &p, &tm, &tz) == 0)
+	tmp = &tm;
+    }
+  if (!tmp)
+    {
+      time_t t = time(NULL);
+      tmp = localtime(&t);
+    }
+  strftime (date, sizeof (date), "%d-%b-%Y %H:%M:%S", tmp);
   util_send ("%s", command->name);
   util_send (" \"%s +0000\"", date);
   return RESP_OK;
@@ -363,11 +366,11 @@ fetch_internaldate (struct fetch_command *command, char **arg)
   Functionally equivalent to BODY.PEEK[HEADER], differing in the syntax of
   the resulting untagged FETCH data (RFC822.HEADER is returned). */
 static int
-fetch_rfc822_header (struct fetch_command *command, char **arg)
+fetch_rfc822_header (struct fetch_command *command, char **arg ARG_UNUSED)
 {
   char buffer[32];
   char *p = buffer;
-  (void)arg; /* No arguments.  */
+
   strcpy (buffer, ".PEEK[HEADER]");
   fetch_body (command, &p);
   return RESP_OK;
@@ -377,11 +380,11 @@ fetch_rfc822_header (struct fetch_command *command, char **arg)
    Functionally equivalent to BODY[TEXT], differing in the syntax of the
    resulting untagged FETCH data (RFC822.TEXT is returned). */
 static int
-fetch_rfc822_text (struct fetch_command *command, char **arg)
+fetch_rfc822_text (struct fetch_command *command, char **arg ARG_UNUSED)
 {
   char buffer[16];
   char *p = buffer;
-  (void)arg; /* No arguments.  */
+
   strcpy (buffer, "[TEXT]");
   fetch_body (command, &p);
   return RESP_OK;
@@ -389,11 +392,11 @@ fetch_rfc822_text (struct fetch_command *command, char **arg)
 
 /* The [RFC-822] size of the message.  */
 static int
-fetch_rfc822_size (struct fetch_command *command, char **arg)
+fetch_rfc822_size (struct fetch_command *command, char **arg ARG_UNUSED)
 {
   size_t size = 0;
   size_t lines = 0;
-  (void)arg; /* No arguments.  */
+  
   message_size (command->msg, &size);
   message_lines (command->msg, &lines);
   util_send ("%s %u", command->name, size + lines);
@@ -444,10 +447,10 @@ fetch_rfc822 (struct fetch_command *command, char **arg)
 
 /* UID: The unique identifier for the message.  */
 static int
-fetch_uid (struct fetch_command *command, char **arg)
+fetch_uid (struct fetch_command *command, char **arg ARG_UNUSED)
 {
   size_t uid = 0;
-  (void)arg; /* No arguments.  */
+
   message_get_uid (command->msg, &uid);
   util_send ("%s %d", command->name, uid);
   return RESP_OK;
@@ -458,9 +461,8 @@ fetch_uid (struct fetch_command *command, char **arg)
    server by parsing the [MIME-IMB] header fields in the [RFC-822] header and
    [MIME-IMB] headers.  */
 static int
-fetch_bodystructure (struct fetch_command *command, char **arg)
+fetch_bodystructure (struct fetch_command *command, char **arg ARG_UNUSED)
 {
-  (void)arg; /* No arguments.  */
   util_send ("%s (", command->name);
   fetch_bodystructure0 (command->msg, 1); /* 1 means with extension data.  */
   util_send (")");
@@ -527,13 +529,15 @@ fetch_send_header_value (header_t header, const char *name,
       util_send_qstring (buffer);
       free (buffer);
     }
+  else if (defval)
+    util_send_qstring (defval);
   else
-    util_send (defval ? defval : "NIL");
+    util_send ("NIL");
 }
 
 static void
 fetch_send_header_list (header_t header, const char *name,
-			 const char *defval, int space)
+			const char *defval, int space)
 {
   char *buffer;
   
@@ -544,8 +548,10 @@ fetch_send_header_list (header_t header, const char *name,
       send_parameter_list (buffer);
       free (buffer);
     }
+  else if (defval)
+    send_parameter_list (defval);
   else
-    util_send (defval ? defval : "NIL");
+    util_send ("NIL");
 }
 
 static void
@@ -562,7 +568,7 @@ fetch_send_header_address (header_t header, const char *name,
       free (buffer);
     }
   else
-    fetch_send_address (defval ? defval : "NIL");
+    fetch_send_address (defval);
 }
 
 /* ENVELOPE:
@@ -880,7 +886,7 @@ bodystructure (message_t msg, int extension)
   else
     {
       /* Default? If Content-Type is not present consider as text/plain.  */
-      util_send ("TEXT PLAIN (\"CHARSET\" \"US-ASCII\")");
+      util_send ("\"TEXT\" \"PLAIN\" (\"CHARSET\" \"US-ASCII\")");
       text_plain = 1;
     }
   
