@@ -1,19 +1,19 @@
-/* GNU mailutils - a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001 Free Software Foundation, Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 1999, 2001, 2002 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.
 
-   This program is distributed in the hope that it will be useful,
+   GNU Mailutils is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with GNU Mailutils; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
 
 #include "imap4d.h"
 
@@ -481,7 +481,8 @@ parse_equiv_key (struct parsebuf *pb)
   if (parse_search_key_list (pb))
     {
       /* shouldn't happen */
-      syslog(LOG_CRIT, "%s:%d: INTERNAL ERROR", __FILE__, __LINE__);
+      syslog(LOG_CRIT, _("%s:%d: INTERNAL ERROR (please report)"),
+	     __FILE__, __LINE__);
       abort (); 
     }
   
@@ -568,7 +569,8 @@ parse_simple_key (struct parsebuf *pb)
 	      put_code (pb, (inst_t) parse_regmem (pb, set));
 	      break;
 	    default:
-	      syslog(LOG_CRIT, "%s:%d: INTERNAL ERROR", __FILE__, __LINE__);
+	      syslog(LOG_CRIT, _("%s:%d: INTERNAL ERROR (please report)"),
+		     __FILE__, __LINE__);
 	      abort (); /* should never happen */
 	    }
 	}  
@@ -635,7 +637,7 @@ _search_pop (struct parsebuf *pb)
 {
   if (pb->tos == 0)
     {
-      syslog(LOG_CRIT, "%s:%d: INTERNAL ERROR", __FILE__, __LINE__);
+      syslog(LOG_CRIT, _("%s:%d: INTERNAL ERROR (please report)"), __FILE__, __LINE__);
       abort (); /* shouldn't happen */
     }
   return pb->stack[--pb->tos];
