@@ -1,19 +1,19 @@
-/* GNU mailutils - a suite of utilities for electronic mail
+/* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU Library General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
 
 #ifndef _MAILUTILS_HEADER_H
 #define _MAILUTILS_HEADER_H
@@ -62,6 +62,9 @@ extern "C" {
 #define MU_HEADER_X_UIDL                    "X-UIDL"
 #define MU_HEADER_X_UID                     "X-UID"
 #define MU_HEADER_X_IMAPBASE                "X-IMAPbase"
+#define MU_HEADER_ENV_SENDER                "X-Envelope-Sender"
+#define MU_HEADER_ENV_DATE                  "X-Envelope-Date"
+#define MU_HEADER_FCC                       "Fcc"
 
 /* Mime support header attribute */
 
@@ -95,37 +98,37 @@ extern int header_lines __P ((header_t, size_t *));
 
 
 extern int header_set_set_value __P ((header_t,
-				      int (*_set_value) __P ((header_t,
-							      const char *,
-							      const char *,
-							      int)), void *));
+				      int (*_set_value) __PMT ((header_t,
+							        const char *,
+							        const char *,
+							        int)), void *));
 
 extern int header_set_get_value __P ((header_t,
-				      int (*_get_value) __P ((header_t,
-							      const char *,
-							      char *,
-							      size_t,
-							      size_t *)),
+				      int (*_get_value) __PMT ((header_t,
+							        const char *,
+							        char *,
+							        size_t,
+							        size_t *)),
 				      void *));
 extern int header_set_get_fvalue __P ((header_t,
-				       int (*_get_value) __P ((header_t,
-							       const char *,
-							       char *,
-							       size_t,
-							       size_t *)),
+				       int (*_get_value) __PMT ((header_t,
+							         const char *,
+							         char *,
+							         size_t,
+							         size_t *)),
 				       void *));
 
 extern int header_set_size __P ((header_t, int (*_size)
-				 __P ((header_t, size_t *)), void *));
+				 __PMT ((header_t, size_t *)), void *));
 
 extern int header_set_lines __P ((header_t,
-				  int (*_lines) __P ((header_t,
-						      size_t *)), void *));
+				  int (*_lines) __PMT ((header_t,
+						        size_t *)), void *));
 
 extern int header_set_fill __P ((header_t,
-				 int (*_fill) __P ((header_t, char *,
-						    size_t, off_t,
-						    size_t *)), void *owner));
+				 int (*_fill) __PMT ((header_t, char *,
+						      size_t, off_t,
+						      size_t *)), void *owner));
 
 #ifdef __cplusplus
 }
