@@ -1,19 +1,19 @@
-/* GNU mailutils - a suite of utilities for electronic mail
+/* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Library Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
 
 #ifndef _MAILUTILS_MUTIL_H
 #define _MAILUTILS_MUTIL_H
@@ -47,6 +47,7 @@ struct mu_timezone
 
 typedef struct mu_timezone mu_timezone;
 
+int mu_parse_date __P ((const char *p, time_t *rettime, const time_t *now));
 extern int mu_parse_imap_date_time __P ((const char **p, struct tm * tm,
 					 mu_timezone * tz));
 extern int mu_parse_ctime_date_time __P ((const char **p, struct tm * tm,
@@ -103,7 +104,7 @@ extern char * mu_getcwd __P((void));
   
 extern int mu_spawnvp(const char* prog, const char* const av[], int* stat);
 
-typedef void *(*mu_retrieve_fp) __P((void *));
+typedef void *(*mu_retrieve_fp) __PMT((void *));
 extern void mu_register_retriever __P((list_t *pflist, mu_retrieve_fp fun));
 extern void * mu_retrieve __P((list_t flist, void *data));
 
@@ -111,6 +112,9 @@ extern int mu_unroll_symlink __P((char *out, size_t outsz, const char *in));
 
 extern char * mu_expand_path_pattern __P((const char *pattern,
 					  const char *username));
+
+extern int mu_rfc2822_references __P((message_t msg, char **pstr));
+extern int mu_rfc2822_in_reply_to __P((message_t msg, char **pstr));
   
 #ifdef __cplusplus
 }
