@@ -1,7 +1,6 @@
 
 /*  A Bison parser, made from msgset.y
- by  GNU Bison version 1.27
-  */
+    by GNU Bison version 1.28  */
 
 #define YYBISON 1  /* Identify Bison output.  */
 
@@ -44,13 +43,15 @@ static int select_body __P ((message_t msg, void *closure));
 static int select_type __P ((message_t msg, void *closure));
 static int select_sender __P ((message_t msg, void *closure));
 static int select_deleted __P ((message_t msg, void *closure));
-
+static int check_set __P ((msgset_t **pset));
+ 
 int yyerror __P ((const char *));
 int yylex  __P ((void));
 
+static int msgset_flags = MSG_NODELETED;
 static msgset_t *result;
 
-#line 56 "msgset.y"
+#line 58 "msgset.y"
 typedef union {
   char *string;
   int number;
@@ -131,10 +132,10 @@ static const short yyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-    71,    75,    79,    83,    87,    91,    95,    99,   105,   106,
-   110,   116,   117,   121,   127,   128,   134,   137,   147,   151,
-   160,   167,   171,   177,   178,   182,   188,   189,   203,   209,
-   210,   218,   222
+    73,    77,    81,    85,    89,    93,    97,   101,   107,   108,
+   112,   118,   124,   128,   134,   135,   141,   144,   154,   158,
+   167,   174,   178,   184,   185,   189,   195,   196,   210,   216,
+   217,   225,   229
 };
 #endif
 
@@ -217,7 +218,7 @@ static const short yycheck[] = {    14,
 };
 /* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
 #line 3 "/usr/share/bison.simple"
-/* This file comes from bison-1.27.  */
+/* This file comes from bison-1.28.  */
 
 /* Skeleton output parser for bison,
    Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.
@@ -430,7 +431,7 @@ __yy_memcpy (char *to, char *from, unsigned int count)
 #endif
 #endif
 
-#line 216 "/usr/share/bison.simple"
+#line 217 "/usr/share/bison.simple"
 
 /* The user can define YYPARSE_PARAM as the name of an argument to be passed
    into yyparse.  The argument should have type void *.
@@ -759,79 +760,87 @@ yyreduce:
   switch (yyn) {
 
 case 1:
-#line 72 "msgset.y"
+#line 74 "msgset.y"
 {
 	     result = msgset_make_1 (cursor);
 	   ;
     break;}
 case 2:
-#line 76 "msgset.y"
+#line 78 "msgset.y"
 {
 	     result = msgset_make_1 (cursor);
 	   ;
     break;}
 case 3:
-#line 80 "msgset.y"
+#line 82 "msgset.y"
 {
 	     result = yyvsp[0].mset;
 	   ;
     break;}
 case 4:
-#line 84 "msgset.y"
+#line 86 "msgset.y"
 {
 	     result = msgset_select (select_deleted, NULL, 0, 1);
 	   ;
     break;}
 case 5:
-#line 88 "msgset.y"
+#line 90 "msgset.y"
 {
 	     result = msgset_select (select_deleted, NULL, 1, 1);
 	   ;
     break;}
 case 6:
-#line 92 "msgset.y"
+#line 94 "msgset.y"
 {
-	     result = msgset_range (1, total);
+	     result = msgset_select (select_deleted, NULL, 0, total);
 	   ;
     break;}
 case 7:
-#line 96 "msgset.y"
+#line 98 "msgset.y"
 {
 	     result = msgset_select (select_deleted, NULL, 1, 1);
 	   ;
     break;}
 case 8:
-#line 100 "msgset.y"
+#line 102 "msgset.y"
 {
 	     result = msgset_select (select_deleted, NULL, 0, 1);
 	   ;
     break;}
 case 10:
-#line 107 "msgset.y"
+#line 109 "msgset.y"
 {
 	     yyval.mset = msgset_append (yyvsp[-2].mset, yyvsp[0].mset);
 	   ;
     break;}
 case 11:
-#line 111 "msgset.y"
+#line 113 "msgset.y"
 {
 	     yyval.mset = msgset_append (yyvsp[-1].mset, yyvsp[0].mset);
 	   ;
     break;}
+case 12:
+#line 119 "msgset.y"
+{
+	     yyval.mset = yyvsp[0].mset;
+	     if (check_set (&yyval.mset))
+	       YYABORT;
+	   ;
+    break;}
 case 13:
-#line 118 "msgset.y"
+#line 125 "msgset.y"
 {
 	     yyval.mset = yyvsp[-1].mset;
 	   ;
     break;}
 case 14:
-#line 122 "msgset.y"
+#line 129 "msgset.y"
 {
 	     yyval.mset = msgset_negate (yyvsp[0].mset);
 	   ;
     break;}
 case 16:
-#line 129 "msgset.y"
+#line 136 "msgset.y"
 {
 	     yyval.mset = msgset_expand (yyvsp[-3].mset, yyvsp[-1].mset);
 	     msgset_free (yyvsp[-3].mset);
@@ -839,7 +848,7 @@ case 16:
 	   ;
     break;}
 case 18:
-#line 138 "msgset.y"
+#line 145 "msgset.y"
 {
 	     struct header_data hd;
 	     hd.header = yyvsp[-1].string;
@@ -851,55 +860,55 @@ case 18:
 	   ;
     break;}
 case 19:
-#line 148 "msgset.y"
+#line 155 "msgset.y"
 {
 	     yyval.mset = msgset_select (select_body, yyvsp[0].string, 0, 0);
 	   ;
     break;}
 case 20:
-#line 152 "msgset.y"
+#line 159 "msgset.y"
 {
 	     if (strchr ("dnorTtu", yyvsp[0].type) == NULL)
 	       {
-		 yyerror ("unknown message type");
+		 yyerror (_("unknown message type"));
 		 YYERROR;
 	       }
 	     yyval.mset = msgset_select (select_type, (void *)&yyvsp[0].type, 0, 0);
 	   ;
     break;}
 case 21:
-#line 161 "msgset.y"
+#line 168 "msgset.y"
 {
 	     yyval.mset = msgset_select (select_sender, (void *)yyvsp[0].string, 0, 0);
 	     free (yyvsp[0].string);
 	   ;
     break;}
 case 22:
-#line 168 "msgset.y"
+#line 175 "msgset.y"
 {
 	     yyval.string = NULL;
 	   ;
     break;}
 case 23:
-#line 172 "msgset.y"
+#line 179 "msgset.y"
 {
 	     yyval.string = yyvsp[0].string;
 	   ;
     break;}
 case 25:
-#line 179 "msgset.y"
+#line 186 "msgset.y"
 {
 	     yyval.mset = msgset_append (yyvsp[-2].mset, yyvsp[0].mset);
 	   ;
     break;}
 case 26:
-#line 183 "msgset.y"
+#line 190 "msgset.y"
 {
 	     yyval.mset = msgset_append (yyvsp[-1].mset, yyvsp[0].mset);
 	   ;
     break;}
 case 28:
-#line 190 "msgset.y"
+#line 197 "msgset.y"
 {
 	     if (yyvsp[0].mset->npart == 1)
 	       {
@@ -915,13 +924,13 @@ case 28:
 	   ;
     break;}
 case 29:
-#line 204 "msgset.y"
+#line 211 "msgset.y"
 {
 	     yyval.mset = msgset_range (yyvsp[-2].number, total);
 	   ;
     break;}
 case 31:
-#line 211 "msgset.y"
+#line 218 "msgset.y"
 {
 	     yyval.mset = msgset_expand (yyvsp[-3].mset, yyvsp[-1].mset);
 	     msgset_free (yyvsp[-3].mset);
@@ -929,20 +938,20 @@ case 31:
 	   ;
     break;}
 case 32:
-#line 219 "msgset.y"
+#line 226 "msgset.y"
 {
 	     yyval.mset = msgset_make_1 (yyvsp[0].number);
 	   ;
     break;}
 case 33:
-#line 223 "msgset.y"
+#line 230 "msgset.y"
 {
 	     yyval.mset = yyvsp[-1].mset;
 	   ;
     break;}
 }
    /* the action file gets copied in in place of this dollarsign */
-#line 542 "/usr/share/bison.simple"
+#line 543 "/usr/share/bison.simple"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1162,7 +1171,7 @@ yyerrhandle:
     }
   return 1;
 }
-#line 227 "msgset.y"
+#line 234 "msgset.y"
 
 
 static int xargc;
@@ -1176,17 +1185,17 @@ yyerror (const char *s)
   fprintf (stderr, "%s: ", xargv[0]);
   fprintf (stderr, "%s", s);
   if (!cur_p)
-    fprintf (stderr, " near end");
+    fprintf (stderr, _(" near end"));
   else if (*cur_p == 0)
     {
       int i =  (*cur_p == 0) ? cur_ind + 1 : cur_ind;
       if (i == xargc)
-	fprintf (stderr, " near end");
+	fprintf (stderr, _(" near end"));
       else
-	fprintf (stderr, " near %s", xargv[i]);
+	fprintf (stderr, _(" near %s"), xargv[i]);
     }
   else
-    fprintf (stderr, " near %s", cur_p);
+    fprintf (stderr, _(" near %s"), cur_p);
   fprintf (stderr, "\n");
   return 0;
 }
@@ -1270,17 +1279,26 @@ yylex()
 }
 
 int
-msgset_parse (const int argc, char **argv, msgset_t **mset)
+msgset_parse (const int argc, char **argv, int flags, msgset_t **mset)
 {
   int rc;
   xargc = argc;
   xargv = argv;
+  msgset_flags = flags;
   cur_ind = 1;
   cur_p = NULL;
   result = NULL;
   rc = yyparse ();
   if (rc == 0)
-    *mset = result;
+    {
+      if (result == NULL)
+	{
+	  util_noapp ();
+	  rc = 1;
+	}
+      else
+	*mset = result;
+    }
   return rc;
 }
 
@@ -1301,12 +1319,23 @@ msgset_free (msgset_t *msg_set)
     }
 }
 
+size_t
+msgset_count (msgset_t *set)
+{
+  size_t count = 0;
+  for (; set; set = set->next)
+    count++;
+  return count;
+}
+
 /* Create a message set consisting of a single msg_num and no subparts */
 msgset_t *
 msgset_make_1 (int number)
 {
   msgset_t *mp;
 
+  if (number == 0)
+    return NULL;
   mp = xmalloc (sizeof (*mp));
   mp->next = NULL;
   mp->npart = 1;
@@ -1381,7 +1410,7 @@ msgset_range (int low, int high)
 
   if (low >= high)
     {
-      yyerror ("range error");
+      yyerror (_("range error"));
       return NULL;
     }
 
@@ -1563,13 +1592,12 @@ select_body (message_t msg, void *closure)
 }
 
 int
-select_sender (message_t msg, void *closure)
+select_sender (message_t msg ARG_UNUSED, void *closure ARG_UNUSED)
 {
   /* char *sender = (char*) closure; */
   /* FIXME: all messages from sender argv[i] */
   /* Annoying we can use address_create() for that
      but to compare against what? The email ?  */
-  (void)msg; (void)closure;
   return 0;
 }
 
@@ -1602,17 +1630,73 @@ select_type (message_t msg, void *closure)
 }
 
 int
-select_deleted (message_t msg, void *closure)
+select_deleted (message_t msg, void *closure ARG_UNUSED)
 {
   attribute_t attr= NULL;
   int rc;
 
-  (void)closure;
   message_get_attribute (msg, &attr);
   rc = attribute_is_deleted (attr);
   return strcmp (xargv[0], "undelete") == 0 ? rc : !rc;
 }
 
+int
+check_set (msgset_t **pset)
+{
+  int flags = msgset_flags;
+  int rc = 0;
+  
+  if (msgset_count (*pset) == 1)
+    flags ^= MSG_SILENT;
+  if (flags & MSG_NODELETED)
+    {
+      msgset_t *p = *pset, *prev = NULL;
+      msgset_t *delset = NULL;
+
+      while (p)
+	{
+	  msgset_t *next = p->next;
+	  if (util_isdeleted (p->msg_part[0]))
+	    {
+	      if ((flags & MSG_SILENT)
+		  && prev != NULL
+		  && next != NULL)
+		{
+		  /* Mark subset as deleted */
+		  p->next = delset;
+		  delset = p;
+		  /* Remove it from the set */
+		  if (prev)
+		    prev->next = next;
+		  else
+		    *pset = next;
+		}
+	      else
+		{
+		  util_error (_("%lu: Inappropriate message (has been deleted)"),
+			      (unsigned long) p->msg_part[0]);
+		  /* Delete entire set */
+		  delset = *pset;
+		  *pset = NULL;
+		  rc = 1;
+		  break;
+		}
+	    }
+	  else
+	    prev = p;
+	  p = next;
+	}
+
+      if (delset)
+	msgset_free (delset);
+
+      if (!*pset)
+	rc = 1;
+    }
+
+  return rc;
+}
+
 #if 0
 void
 msgset_print (msgset_t *mset)
