@@ -1,19 +1,19 @@
-/* GNU mailutils - a suite of utilities for electronic mail
+/* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Library Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU Library General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -129,18 +129,37 @@ list_count (list_t list, size_t *pcount)
   return 0;
 }
 
+list_comparator_t
+list_set_comparator (list_t list, list_comparator_t comp)
+{
+  list_comparator_t old_comp;
+  if (list == NULL)
+    return NULL;
+  old_comp = list->comp;
+  list->comp = comp;
+  return old_comp;
+}  
+
+static int
+def_comp (const void *item, const void *value)
+{
+  return item != value;
+}
+
 int
 list_remove (list_t list, void *item)
 {
   struct list_data *current, *previous;
+  list_comparator_t comp;
   int status = ENOENT;
   if (list == NULL)
     return EINVAL;
+  comp = list->comp ? list->comp : def_comp;
   monitor_wrlock (list->monitor);
   for (previous = &(list->head), current = list->head.next;
        current != &(list->head); previous = current, current = current->next)
     {
-      if (current->item == item)
+      if (comp (current->item, item) == 0)
 	{
 	  previous->next = current->next;
 	  current->next->prev = previous;
@@ -158,14 +177,17 @@ int
 list_replace (list_t list, void *old_item, void *new_item)
 {
   struct list_data *current, *previous;
+  list_comparator_t comp;
   int status = ENOENT;
+  
   if (list == NULL)
     return EINVAL;
+  comp = list->comp ? list->comp : def_comp;
   monitor_wrlock (list->monitor);
   for (previous = &(list->head), current = list->head.next;
        current != &(list->head); previous = current, current = current->next)
     {
-      if (current->item == old_item)
+      if (comp (current->item, old_item) == 0)
 	{
 	  current->item = new_item;
 	  status = 0;
