@@ -17,7 +17,7 @@
 
 #include <mh.h>
 
-const char *argp_program_version = "install-mh (" PACKAGE_STRING ")";
+const char *program_version = "install-mh (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU MH install-mh\v"
 "Use -help to obtain the list of traditional MH options.");
 static char args_doc[] = "";
@@ -26,6 +26,8 @@ static char args_doc[] = "";
 static struct argp_option options[] = {
   {"auto",  ARG_AUTO, NULL, 0, N_("Do not ask for anything")},
   {"compat", ARG_COMPAT, NULL, OPTION_HIDDEN, ""},
+  {"license", ARG_LICENSE, 0,      0,
+   N_("Display software license"), -1},
   {NULL}
 };
 
@@ -49,6 +51,10 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
     case ARG_COMPAT:
       break;
 
+    case ARG_LICENSE:
+      mh_license (argp_program_version);
+      break;
+
     default:
       return 1;
     }
@@ -60,9 +66,11 @@ main (int argc, char **argv)
 {
   char *home, *name;
   extern int mh_auto_install;
+  
   /* Native Language Support */
   mu_init_nls ();
 
+  mu_argp_init (program_version, NULL);
   mh_auto_install = 0;
   mh_argp_parse (argc, argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, NULL);
