@@ -17,7 +17,7 @@
 
 #include <mh.h>
 
-const char *argp_program_version = "mark (" PACKAGE_STRING ")";
+const char *program_version = "mark (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU MH mark\v"
 "Use -help to obtain the list of traditional MH options.");
 static char args_doc[] = "[msgs...]";
@@ -40,6 +40,8 @@ static struct argp_option options[] = {
   {"zero", ARG_ZERO, N_("BOOL"), OPTION_ARG_OPTIONAL,
    N_("Empty the sequence before adding messages")},
   {"nozero", ARG_NOZERO, NULL, OPTION_HIDDEN, "" },
+  {"license", ARG_LICENSE, 0,      0,
+   N_("Display software license"), -1},
   {NULL}
 };
 
@@ -76,7 +78,6 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
 {
   switch (key)
     {
-    case '+':
     case ARG_FOLDER: 
       current_folder = arg;
       break;
@@ -113,6 +114,10 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       seq_flags &= ~SEQ_ZERO;
       break;
       
+    case ARG_LICENSE:
+      mh_license (argp_program_version);
+      break;
+
     default:
       return 1;
     }
@@ -188,6 +193,7 @@ main (int argc, char **argv)
   url_t url;
   
   mu_init_nls ();
+  mu_argp_init (program_version, NULL);
   mh_argp_parse (argc, argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, &index);
 
