@@ -1,19 +1,19 @@
-/* GNU mailutils - a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 1999, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.
 
-   This program is distributed in the hope that it will be useful,
+   GNU Mailutils is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with GNU Mailutils; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
 
 /* This module implements execution of MH format strings. */
 
@@ -23,6 +23,10 @@
 # include <strings.h>
 #endif
 
+#define obstack_chunk_alloc malloc
+#define obstack_chunk_free free
+#include <obstack.h>
+
 typedef struct       /* A string object type */
 {
   int size;          /* Allocated size or 0 for static storage */
@@ -42,10 +46,11 @@ struct mh_machine
   mh_instr_t *prog;         /* Program itself */
   int stop;                 /* Stop execution immediately */
 
-  char *outbuf;             /* Output buffer */
-  size_t width;             /* Output buffer size */
+  struct obstack stk;       /* Output buffer */
+  size_t width;             /* Output buffer width */
   size_t ind;               /* Output buffer index */
 
+  list_t addrlist;          /* The list of email addresses output this far */
   int fmtflags;             /* Current formatting flags */
   
   message_t message;        /* Current message */
@@ -163,45 +168,153 @@ compress_ws (char *str, size_t *size)
   *size = p - (unsigned char*) str;
 }
 
+static void
+put_string (struct mh_machine *mach, char *str, int len)
+{
+  if (len == 0)
+    len = strlen (str);
+  obstack_grow (&mach->stk, str, len);
+  mach->ind += len;
+}
+
+static void
+print_hdr_segment (struct mh_machine *mach, char *str, size_t len)
+{
+  if (!len)
+    len = strlen (str);
+
+  if (len < mach->width)
+    put_string (mach, str, len);
+  else
+    {
+      char *endp = str + len;
+      
+      while (str < endp)
+	{
+	  size_t rest;
+	  char *p;
+	  size_t size;
+	  
+	  size = endp - str;
+	  rest = mach->width - mach->ind;
+	  if (size < rest)
+	    {
+	      put_string (mach, str, size);
+	      break;
+	    }
+	  
+	  for (p = str + rest - 1; p > str && !isspace (*p); p--)
+	    ;
+
+	  if (p > str)
+	    {
+	      put_string (mach, str, p - str);
+	      put_string (mach, "\n\t", 0);
+	      mach->ind = 8;
+	      str = p;
+	    }
+	  else
+	    {
+	      put_string (mach, str, size);
+	      str += len;
+	    }
+	}
+    }
+}
+
 /* Print len bytes from str into mach->outbuf */
 static void
-print_string (struct mh_machine *mach, size_t width, char *str, size_t fmtlen)
+print_hdr_string (struct mh_machine *mach, char *str)
 {
-  size_t rest, len;
+  char *p;
+  
+  if (!str)
+    str = "";
+
+  p = strchr (str, '\n');
+  while (p)
+    {
+      print_hdr_segment (mach, str, p - str + 1);
+      mach->ind = 0;
+      str = p + 1;
+      p = strchr (str, '\n');
+    }
+    
+  if (str[0])
+    print_hdr_segment (mach, str, 0);
+}
+
+static void
+print_simple_segment (struct mh_machine *mach, size_t width,
+		      char *str, size_t len)
+{
+  size_t rest;
+
+  if (!str)
+    str = "";
+
+  if (!len)
+    len = strlen (str);
+
+  if (!width)
+    width = mach->width;
+
+  rest = width - mach->ind;
+  if (rest == 0)
+    return;
+  if (len > rest)
+    len = rest;
+
+  put_string (mach, str, len);
+}
+
+static void
+print_string (struct mh_machine *mach, size_t width, char *str)
+{
+  char *p;
   
   if (!str)
     str = "";
 
   if (!width)
     width = mach->width;
-  len = strlen (str);
-  rest = width - mach->ind;
-  if (len > rest)
-    {
-      if (fmtlen >= len)
-	fmtlen = rest;
-      len = rest;
-    }
-  
-  if (fmtlen < len)
-    len = fmtlen;
-  
-  memcpy (mach->outbuf + mach->ind, str, len);
-  mach->ind += len;
 
-  if (fmtlen > len)
+  p = strchr (str, '\n');
+  while (p)
     {
-      fmtlen -= len;
-      memset (mach->outbuf + mach->ind, ' ', fmtlen);
-      mach->ind += fmtlen;
+      print_simple_segment (mach, width, str, p - str + 1);
+      mach->ind = 0;
+      str = p + 1;
+      p = strchr (str, '\n');
+    }
+    
+  if (str[0])
+    print_simple_segment (mach, width, str, 0);
+}
+  
+static void
+print_fmt_string (struct mh_machine *mach, size_t fmtwidth, char *str)
+{
+  size_t len = strlen (str);
+  if (len > fmtwidth)
+    len = fmtwidth;
+  put_string (mach, str, len);
+
+  if (fmtwidth > len)
+    {
+      fmtwidth -= len;
+      mach->ind += fmtwidth;
+      while (fmtwidth--)
+	obstack_1grow (&mach->stk, ' ');
     }
 }
 
+/* FIXME: width? */
 static void
 print_obj (struct mh_machine *mach, size_t width, strobj_t *obj)
 {
   if (!strobj_is_null (obj))
-    print_string (mach, 0, strobj_ptr (obj), strobj_len (obj));
+    print_string (mach, 0, strobj_ptr (obj));
 }
 
 static void
@@ -210,11 +323,114 @@ reset_fmt_defaults (struct mh_machine *mach)
   mach->fmtflags = 0;
 }
 
+static void
+format_num (struct mh_machine *mach, long num)
+{
+  int n;
+  char buf[64];
+  char *ptr;
+  int fmtwidth = mach->fmtflags & MH_WIDTH_MASK;
+  int padchar = mach->fmtflags & MH_FMT_ZEROPAD ? '0' : ' ';
+	    
+  n = snprintf (buf, sizeof buf, "%ld", num);
+
+  if (fmtwidth)
+    {
+      if (n > fmtwidth)
+	{
+	  ptr = buf + n - fmtwidth;
+	  *ptr = '?';
+	}
+      else
+	{
+	  int i;
+	  ptr = buf;
+	  for (i = n; i < fmtwidth && mach->ind < mach->width;
+	       i++, mach->ind++)
+	    obstack_1grow (&mach->stk, padchar);
+	}
+    }
+  else
+    ptr = buf;
+
+  print_string (mach, 0, ptr);
+  reset_fmt_defaults (mach);
+}
+
+static void
+format_str (struct mh_machine *mach, char *str)
+{
+  if (!str)
+    str = "";
+  if (mach->fmtflags)
+    {
+      int len = strlen (str);
+      int fmtwidth = mach->fmtflags & MH_WIDTH_MASK;
+      int padchar = ' ';
+				
+      if (mach->fmtflags & MH_FMT_RALIGN)
+	{
+	  int i, n;
+	  
+	  n = fmtwidth - len;
+	  for (i = 0; i < n && mach->ind < mach->width;
+	       i++, mach->ind++, fmtwidth--)
+	    obstack_1grow (&mach->stk, padchar);
+	}
+	      
+      print_fmt_string (mach, fmtwidth, str);
+      reset_fmt_defaults (mach);
+    }
+  else
+    print_string (mach, 0, str);
+}
+
+static int
+addr_cmp (void *item, void *data)
+{
+  address_t a = item;
+  address_t b = data;
+  size_t i, count;
+  int rc = 0;
+  
+  address_get_count (a, &count);
+  for (i = 1; rc == 0 && i <= count; i++)
+    {
+      char *str;
+      if (address_aget_email (a, i, &str))
+	continue;
+      rc = address_contains_email (b, str);
+      free (str);
+    }
+  return rc;
+}
+
+static int
+addrlist_lookup (list_t list, address_t addr)
+{
+  return list_do (list, addr_cmp, addr);
+}
+
+static int
+addr_free (void *item, void *data)
+{
+  address_t addr = item;
+  address_destroy (&addr);
+  return 0;
+}
+
+static void
+addrlist_destroy (list_t *list)
+{
+  list_do (*list, addr_free, NULL);
+  list_destroy (list);
+}
+
 /* Execute pre-compiled format on message msg with number msgno.
    buffer and bufsize specify output storage */
 int
 mh_format (mh_format_t *fmt, message_t msg, size_t msgno,
-	   char *buffer, size_t bufsize)
+	   size_t width, char **pret)
 {
   struct mh_machine mach;
   char buf[64];
@@ -226,9 +442,11 @@ mh_format (mh_format_t *fmt, message_t msg, size_t msgno,
   mach.message = msg;
   mach.msgno = msgno;
   
-  mach.width = bufsize - 1;  /* Allow for terminating NULL */
-  mach.outbuf = buffer;
+  mach.width = width;
   mach.pc = 1;
+  obstack_init (&mach.stk);
+  list_create (&mach.addrlist);
+
   reset_fmt_defaults (&mach);
   
   while (!mach.stop)
@@ -351,67 +569,17 @@ mh_format (mh_format_t *fmt, message_t msg, size_t msgno,
   
 	  /* Convert arg_num to arg_str */
 	case mhop_num_to_str:
-	  snprintf (buf, sizeof buf, "%l", mach.arg_num);
+	  snprintf (buf, sizeof buf, "%lu", mach.arg_num);
 	  strobj_free (&mach.arg_str);
 	  strobj_create (&mach.arg_str, buf);
 	  break;
 
 	case mhop_num_print:
-	  {
-	    int n;
-	    char buf[64];
-	    char *ptr;
-	    int fmtwidth = mach.fmtflags & MH_WIDTH_MASK;
-	    int padchar = mach.fmtflags & MH_FMT_ZEROPAD ? '0' : ' ';
-	    
-	    n = snprintf (buf, sizeof buf, "%d", mach.reg_num);
-
-	    if (fmtwidth)
-	      {
-		if (n > fmtwidth)
-		  {
-		    ptr = buf + n - fmtwidth;
-		    *ptr = '?';
-		  }
-		else
-		  {
-		    int i;
-		    ptr = buf;
-		    for (i = n; i < fmtwidth && mach.ind < mach.width;
-			 i++, mach.ind++)
-		      mach.outbuf[mach.ind] = padchar;
-		  }
-	      }
-	    else
-	      ptr = buf;
-
-	    print_string (&mach, 0, ptr, strlen (ptr));
-	    reset_fmt_defaults (&mach);
-	  }
+	  format_num (&mach, mach.reg_num);
 	  break;
 	  
 	case mhop_str_print:
-	  if (mach.fmtflags)
-	    {
-	      int len = strobj_len (&mach.reg_str);
-	      int fmtwidth = mach.fmtflags & MH_WIDTH_MASK;
-	      int padchar = ' ';
-				
-	      if (mach.fmtflags & MH_FMT_RALIGN)
-		{
-		  int i, n;
-
-		  n = fmtwidth - len;
-		  for (i = 0; i < n && mach.ind < mach.width;
-		       i++, mach.ind++, fmtwidth--)
-		    mach.outbuf[mach.ind] = padchar;
-		}
-		
-	      print_string (&mach, 0, strobj_ptr (&mach.reg_str), fmtwidth);
-	      reset_fmt_defaults (&mach);
-	    }
-	  else
-	    print_obj (&mach, 0, &mach.reg_str);
+	  format_str (&mach, strobj_ptr (&mach.reg_str));
 	  break;
 
 	case mhop_fmtspec:
@@ -419,16 +587,40 @@ mh_format (mh_format_t *fmt, message_t msg, size_t msgno,
 	  break;
 
 	default:
-	  mh_error ("Unknown opcode: %x", opcode);
+	  mh_error (_("INTERNAL ERROR: Unknown opcode: %x"), opcode);
 	  abort ();
 	}
     }
   strobj_free (&mach.reg_str);
   strobj_free (&mach.arg_str);
-  mach.outbuf[mach.ind] = 0;
+  addrlist_destroy (&mach.addrlist);
+  
+  if (pret)
+    {
+      obstack_1grow (&mach.stk, 0);
+      *pret = strdup (obstack_finish (&mach.stk));
+    }
+  obstack_free (&mach.stk, NULL);
   return mach.ind;
 }
 
+int
+mh_format_str (mh_format_t *fmt, char *str, size_t width, char **pret)
+{
+  message_t msg = NULL;
+  header_t hdr = NULL;
+  int rc;
+  
+  if (message_create (&msg, NULL))
+    return -1;
+  message_get_header (msg, &hdr);
+  header_set_value (hdr, "text", str, 1);
+  rc = mh_format (fmt, msg, 1, width, pret);
+  message_destroy (&msg, NULL);
+  return rc;
+}
+  
+
 void
 mh_format_dump (mh_format_t *fmt)
 {
@@ -679,8 +871,8 @@ static void
 builtin_me (struct mh_machine *mach)
 {
   char *s = mh_my_email ();
-  strobj_free (&mach->arg_str);
-  strobj_create (&mach->arg_str, s);
+  strobj_realloc (&mach->arg_str, strlen (s) + 3);
+  sprintf (strobj_ptr (&mach->arg_str), "<%s>", s);
 }
 
 static void
@@ -733,7 +925,7 @@ builtin_divide (struct mh_machine *mach)
 {
   if (!mach->arg_num)
     {
-      mh_error ("format: divide by zero");
+      mh_error (_("format: divide by zero"));
       mach->stop = 1;
     }
   else
@@ -745,7 +937,7 @@ builtin_modulo (struct mh_machine *mach)
 {
   if (!mach->arg_num)
     {
-      mh_error ("format: divide by zero");
+      mh_error (_("format: divide by zero"));
       mach->stop = 1;
     }
   else
@@ -843,16 +1035,14 @@ builtin_trim (struct mh_machine *mach)
 static void
 builtin_putstr (struct mh_machine *mach)
 {
-  print_string (mach, 0,
-		strobj_ptr (&mach->arg_str), strobj_len (&mach->arg_str));
+  print_string (mach, 0, strobj_ptr (&mach->arg_str));
 }
 
 /*     putstrf    expr              print str in a fixed width*/
 static void
 builtin_putstrf (struct mh_machine *mach)
 {
-  print_string (mach, mach->reg_num,
-		strobj_ptr (&mach->arg_str), strobj_len (&mach->arg_str));
+  format_str (mach, strobj_ptr (&mach->arg_str));
 }
 
 /*     putnum     expr              print num*/
@@ -861,7 +1051,7 @@ builtin_putnum (struct mh_machine *mach)
 {
   char *p;
   asprintf (&p, "%d", mach->arg_num);
-  print_string (mach, 0, p, strlen (p));
+  print_string (mach, 0, p);
   free (p);
 }
 
@@ -869,10 +1059,7 @@ builtin_putnum (struct mh_machine *mach)
 static void
 builtin_putnumf (struct mh_machine *mach)
 {
-  char *p;
-  asprintf (&p, "%d", mach->arg_num);
-  print_string (mach, mach->reg_num, p, strlen (p));
-  free (p);
+  format_num (mach, mach->arg_num);
 }
 
 static int
@@ -1518,56 +1705,221 @@ builtin_gname (struct mh_machine *mach)
 static void
 builtin_formataddr (struct mh_machine *mach)
 {
-  address_t addr;
-  size_t num = 0, n, i, length = 0, reglen;
-  char buf[80], *p;
+  address_t addr, dest;
+  size_t size;
   
-  if (address_create (&addr, strobj_ptr (&mach->arg_str)))
+  if (strobj_len (&mach->reg_str) == 0)
+    dest = NULL;
+  else if (address_create (&dest, strobj_ptr (&mach->reg_str)))
     return;
-  address_get_count (addr, &num);
-  for (i = 1; i <= num; i++)
+    
+  if (address_create (&addr, strobj_ptr (&mach->arg_str)))
     {
-      if (address_get_personal (addr, i, buf, sizeof buf, &n) == 0
-	  && n != 0)
-	length += n+1; /* + space */
-      address_get_email (addr, i, buf, sizeof buf, &n);
-      length += n+3; /* two brackets + (eventual) comma */
+      address_destroy (&dest);
+      return;
     }
 
-  if (strobj_is_null (&mach->reg_str))
-    reglen = 0;
-  else
-    reglen = strobj_len (&mach->reg_str);
-  length += reglen + 1;
-  strobj_realloc (&mach->reg_str, length);
-  p = strobj_ptr (&mach->reg_str) + reglen;
-  for (i = 1; i <= num; i++)
+  if (addrlist_lookup (mach->addrlist, addr))
     {
-      if (reglen > 0)
-	*p++ = ',';
-      if (address_get_personal (addr, i, buf, sizeof buf, &n) == 0 && n > 0)
+      address_destroy (&dest);
+      address_destroy (&addr);
+      return;
+    }
+  
+  if (rcpt_mask & RCPT_ME)
+    address_union (&dest, addr);
+  else
+    {
+      int i;
+      size_t num;
+      char *buf;
+      
+      address_get_count (addr, &num);
+      for (i = 1; i <= num; i++)
 	{
-	  memcpy (p, buf, n);
-	  p += n;
-	  *p++ = ' ';
+	  if (address_aget_email (addr, i, &buf) == 0)
+	    {
+	      if (!mh_is_my_name (buf))
+		{
+		  address_t subaddr;
+		  address_get_nth (addr, i, &subaddr);
+		  address_union (&dest, subaddr);
+		  address_destroy (&subaddr);
+		}
+	      free (buf);
+	    }
 	}
-      address_get_email (addr, i, buf, sizeof buf, &n);
-      *p++ = '<';
-      memcpy (p, buf, n);
-      p += n;
-      *p++ = '>';
+    }
+  list_append (mach->addrlist, addr);
+  if (address_to_string (dest, NULL, 0, &size) == 0)
+    {
+      strobj_realloc (&mach->reg_str, size + 1);
+      address_to_string (dest, strobj_ptr (&mach->reg_str), size + 1, NULL);
+      address_destroy (&dest);
     }
-  *p = 0;
 }
 
 /*      putaddr    literal        print str address list with
                                   arg as optional label;
-                                  get line width from num */
+                                  get line width from num
+   FIXME: Currently it's the same as puthdr. Possibly it should do
+   some address-checking as well.
+*/
 static void
 builtin_putaddr (struct mh_machine *mach)
 {
-  print_obj (mach, mach->reg_num, &mach->arg_str);
-  print_obj (mach, mach->reg_num, &mach->reg_str);
+  if (!strobj_is_null (&mach->arg_str))
+    print_hdr_string (mach, strobj_ptr (&mach->arg_str));
+  if (!strobj_is_null (&mach->reg_str))
+    print_hdr_string (mach, strobj_ptr (&mach->reg_str));
+}
+
+/* GNU extension: Strip leading whitespace and eventual Re: (or Re\[[0-9]+\]:)
+   prefix from the argument */
+static void
+builtin_unre (struct mh_machine *mach)
+{
+  char *start, *p;
+
+  for (p = strobj_ptr (&mach->arg_str); *p; p++)
+    if (!isspace (*p))
+      break;
+
+  start = p;
+  
+  if (strncasecmp (p, "re", 2) == 0)
+    {
+      if (p[2] == ':')
+	p += 3;
+      else if (p[2] == '[')
+	{
+	  for (p += 3; *p; p++)
+	    if (*p == ']' || !isdigit (*p))
+	      break;
+	  if (*p == ']' && p[1] == ':') 
+	    p += 2;
+	  else
+	    p = start;
+	}
+      else
+	p = start;
+    }
+
+  if (p != strobj_ptr (&mach->arg_str))
+    {
+      for (; *p && isspace (*p); p++)
+	;
+      
+      p = strdup (p);
+      strobj_free (&mach->arg_str);
+      strobj_create (&mach->arg_str, p);
+      free (p);
+    }
+}  
+
+int
+mh_decode_rcpt_flag (const char *arg)
+{
+  if (strcmp (arg, "to") == 0)
+    return RCPT_TO;
+  else if (strcmp (arg, "cc") == 0)
+    return RCPT_CC;
+  else if (strcmp (arg, "me") == 0)
+    return RCPT_ME;
+  else if (strcmp (arg, "all") == 0)
+    return RCPT_ALL;
+
+  return RCPT_NONE;
+}
+
+static void
+builtin_rcpt (struct mh_machine *mach)
+{
+  int rc = mh_decode_rcpt_flag (strobj_ptr (&mach->arg_str));
+  if (rc == RCPT_NONE)
+    {
+      mh_error (_("Invalid recipient mask"));
+      /* try to continue anyway */
+    }
+  mach->arg_num = rc & rcpt_mask;
+}
+
+static void
+builtin_concat (struct mh_machine *mach)
+{
+  size_t size = strobj_len (&mach->arg_str);
+
+  if (size == 0)
+    return;
+  compress_ws (strobj_ptr (&mach->arg_str), &size);
+  if (strobj_len (&mach->reg_str) == 0)
+    strobj_copy (&mach->reg_str, &mach->arg_str);
+  else
+    {
+      int length = 1;
+    
+      length += 1 + strobj_len (&mach->reg_str); /* reserve en extra space */
+      length += strobj_len (&mach->arg_str);
+      strobj_realloc (&mach->reg_str, length);
+      strcat (strcat (strobj_ptr (&mach->reg_str), " "),
+	      strobj_ptr (&mach->arg_str));
+    }
+}
+
+static void
+builtin_printhdr (struct mh_machine *mach)
+{
+  if (!strobj_is_null (&mach->arg_str))
+    print_hdr_string (mach, strobj_ptr (&mach->arg_str));
+  if (!strobj_is_null (&mach->reg_str))
+    print_hdr_string (mach, strobj_ptr (&mach->reg_str));
+}
+
+static void
+builtin_in_reply_to (struct mh_machine *mach)
+{
+  char *value;
+
+  strobj_free (&mach->arg_str);
+  if (mu_rfc2822_in_reply_to (mach->message, &value))
+    {
+      strobj_create (&mach->arg_str, value);
+      free (value);
+    }
+}
+
+static void
+builtin_references (struct mh_machine *mach)
+{
+  char *value;
+
+  strobj_free (&mach->arg_str);
+  if (mu_rfc2822_references (mach->message, &value) == 0)
+    {
+      strobj_create (&mach->arg_str, value);
+      free (value);
+    }
+}
+
+static void
+builtin_package (struct mh_machine *mach)
+{
+  strobj_free (&mach->arg_str);
+  strobj_set (&mach->arg_str, PACKAGE);
+}
+
+static void
+builtin_package_string (struct mh_machine *mach)
+{
+  strobj_free (&mach->arg_str);
+  strobj_set (&mach->arg_str, PACKAGE_STRING);
+}
+
+static void
+builtin_version (struct mh_machine *mach)
+{
+  strobj_free (&mach->arg_str);
+  strobj_set (&mach->arg_str, VERSION);
 }
 
 /* Builtin function table */
@@ -1592,20 +1944,20 @@ mh_builtin_t builtin_tab[] = {
   { "divide",   builtin_divide,   mhtype_num,  mhtype_num },
   { "modulo",   builtin_modulo,   mhtype_num,  mhtype_num },
   { "num",      builtin_num,      mhtype_num,  mhtype_num },
-  { "lit",      builtin_lit,      mhtype_str,  mhtype_str, 1 },
+  { "lit",      builtin_lit,      mhtype_str,  mhtype_str,  MHA_OPT_CLEAR },
   { "getenv",   builtin_getenv,   mhtype_str,  mhtype_str },
   { "profile",  builtin_profile,  mhtype_str,  mhtype_str },
-  { "nonzero",  builtin_nonzero,  mhtype_num,  mhtype_num, 1 },
-  { "zero",     builtin_zero,     mhtype_num,  mhtype_num, 1 },
-  { "null",     builtin_null,     mhtype_num,  mhtype_str, 1 },
-  { "nonnull",  builtin_nonnull,  mhtype_num,  mhtype_str, 1 },
-  { "comp",     builtin_comp,     mhtype_num,  mhtype_str },
-  { "compval",  builtin_compval,  mhtype_num,  mhtype_str },
-  { "trim",     builtin_trim,     mhtype_str,  mhtype_str, 1 },
-  { "putstr",   builtin_putstr,   mhtype_str,  mhtype_str, 1 },
-  { "putstrf",  builtin_putstrf,  mhtype_str,  mhtype_str, 1 },
-  { "putnum",   builtin_putnum,   mhtype_str,  mhtype_num, 1 },
-  { "putnumf",  builtin_putnumf,  mhtype_str,  mhtype_num, 1 },
+  { "nonzero",  builtin_nonzero,  mhtype_num,  mhtype_num,  MHA_OPTARG },
+  { "zero",     builtin_zero,     mhtype_num,  mhtype_num,  MHA_OPTARG },
+  { "null",     builtin_null,     mhtype_num,  mhtype_str,  MHA_OPTARG },
+  { "nonnull",  builtin_nonnull,  mhtype_num,  mhtype_str,  MHA_OPTARG },
+  { "comp",     builtin_comp,     mhtype_num,  mhtype_str,  MHA_OPTARG },
+  { "compval",  builtin_compval,  mhtype_num,  mhtype_str },	   
+  { "trim",     builtin_trim,     mhtype_str,  mhtype_str,  MHA_OPTARG },
+  { "putstr",   builtin_putstr,   mhtype_none,  mhtype_str, MHA_OPTARG },
+  { "putstrf",  builtin_putstrf,  mhtype_none,  mhtype_str, MHA_OPTARG },
+  { "putnum",   builtin_putnum,   mhtype_none,  mhtype_num, MHA_OPTARG },
+  { "putnumf",  builtin_putnumf,  mhtype_none,  mhtype_num, MHA_OPTARG },
   { "sec",      builtin_sec,      mhtype_num,  mhtype_str },
   { "min",      builtin_min,      mhtype_num,  mhtype_str },
   { "hour",     builtin_hour,     mhtype_num,  mhtype_str },
@@ -1643,8 +1995,17 @@ mh_builtin_t builtin_tab[] = {
   { "path",     builtin_path,     mhtype_str,  mhtype_str },
   { "ingrp",    builtin_ingrp,    mhtype_num,  mhtype_str },
   { "gname",    builtin_gname,    mhtype_str,  mhtype_str},
-  { "formataddr", builtin_formataddr, mhtype_none, mhtype_str, 1 },
+  { "formataddr", builtin_formataddr, mhtype_none, mhtype_str, MHA_OPTARG },
   { "putaddr",  builtin_putaddr,  mhtype_none, mhtype_str },
+  { "unre",     builtin_unre,     mhtype_str,  mhtype_str },
+  { "rcpt",     builtin_rcpt,     mhtype_num,  mhtype_str },
+  { "concat",   builtin_concat,   mhtype_none, mhtype_str,     MHA_OPTARG },
+  { "printhdr", builtin_printhdr, mhtype_none, mhtype_str },
+  { "in_reply_to", builtin_in_reply_to, mhtype_str,  mhtype_none },
+  { "references", builtin_references, mhtype_str,  mhtype_none },
+  { "package",  builtin_package,  mhtype_str, mhtype_none },
+  { "package_string",  builtin_package_string,  mhtype_str, mhtype_none },
+  { "version",  builtin_version,  mhtype_str, mhtype_none },
   { 0 }
 };
 
