@@ -28,7 +28,7 @@
 
 #include <dirent.h>
 
-const char *argp_program_version = "rmf (" PACKAGE_STRING ")";
+const char *program_version = "rmf (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU MH rmf\v"
 "Use -help to obtain the list of traditional MH options.");
 static char args_doc[] = N_("[+folder]");
@@ -39,8 +39,12 @@ static struct argp_option options[] = {
    N_("Specify the folder to delete")},
   {"interactive", ARG_INTERACTIVE, N_("BOOL"), OPTION_ARG_OPTIONAL,
     N_("Interactive mode: ask for confirmation before removing each folder")},
+  {"nointeractive", ARG_NOINTERACTIVE, NULL, OPTION_HIDDEN, ""},
   {"recursive", ARG_RECURSIVE, NULL, 0,
    N_("Recursively delete all subfolders")},
+  {"norecursive", ARG_NORECURSIVE, NULL, OPTION_HIDDEN, ""},
+  {"license", ARG_LICENSE, 0,      0,
+   N_("Display software license"), -1},
   { 0 }
 };
 
@@ -59,7 +63,6 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
 {
   switch (key)
     {
-    case '+':
     case ARG_FOLDER:
       explicit_folder = 1;
       current_folder = arg;
@@ -69,10 +72,22 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       interactive = is_true (arg);
       break;
 
+    case ARG_NOINTERACTIVE:
+      interactive = 0;
+      break;
+	
     case ARG_RECURSIVE:
       recurse = is_true (arg);
       break;
       
+    case ARG_NORECURSIVE:
+      recurse = 0;
+      break;
+
+    case ARG_LICENSE:
+      mh_license (argp_program_version);
+      break;
+
     default:
       return 1;
     }
@@ -134,6 +149,7 @@ main (int argc, char **argv)
   /* Native Language Support */
   mu_init_nls ();
 
+  mu_argp_init (program_version, NULL);
   mh_argp_parse (argc, argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, NULL);
   if (!explicit_folder)
