@@ -20,9 +20,8 @@
 /* AUTH is not yet implemented */
 
 int
-pop3d_auth (const char *arg)
+pop3d_auth (const char *arg ARG_UNUSED)
 {
-  (void)arg;
   if (state != AUTHORIZATION)
     return ERR_WRONG_STATE;
   return ERR_NOT_IMPL;
