@@ -1,19 +1,19 @@
-/* GNU mailutils - a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001 Free Software Foundation, Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 1999, 2001, 2003 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.
 
-   This program is distributed in the hope that it will be useful,
+   GNU Mailutils is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with GNU Mailutils; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
 
 #include "pop3d.h"
 
@@ -41,6 +41,12 @@ pop3d_capa (const char *arg)
   pop3d_outf ("UIDL\r\n");
   pop3d_outf ("RESP-CODES\r\n");
   pop3d_outf ("PIPELINING\r\n");
+
+#ifdef WITH_TLS
+  if (tls_available && tls_done == 0)
+    pop3d_outf ("STLS\r\n");
+#endif /* WITH_TLS */
+
   /* FIXME: This can be Implemented by setting an header field on the
      message.  */
   /*pop3d_outf ("EXPIRE NEVER\r\n"); */
